﻿using System.Linq;
using System.Windows.Controls;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Data;
using System.ServiceModel.DomainServices.Client;
using Telerik.Windows.Controls.DomainServices;
using SilverlightApplication1.Web.Services;
using System;

namespace SilverlightApplication1
{
	public partial class MainPage : UserControl
	{
		private RadObservableCollection<string> distinctValues = new RadObservableCollection<string>();
		private DateTime startTime;

		public MainPage()
		{
			InitializeComponent();
		}
		
		private void OnRadGridViewDistinctValuesLoading(object sender, GridViewDistinctValuesLoadingEventArgs e)
		{
			this.distinctValues.Clear();
			e.ItemsSource = this.distinctValues;

			this.distinctValuesDataSource.QueryParameters[0].Value = e.Column.GetDataMemberName();
			this.distinctValuesDataSource.Load();
		}

		private void OnDistinctValuesDataSourceLoadingData(object sender, LoadingDataEventArgs e)
		{
			e.LoadBehavior = LoadBehavior.RefreshCurrent;
		}

		private void OnDistinctValuesDataSourceLoadedData(object sender, Telerik.Windows.Controls.DomainServices.LoadedDataEventArgs e)
		{
			if (e.HasError)
			{
				e.MarkErrorAsHandled();
			}
			else
			{
				this.distinctValues.AddRange(e.Entities.Cast<DistinctValue>().Select(dv => dv.Value));
			}
		}

		private void OnCustomersDataSourceLoadingData(object sender, Telerik.Windows.Controls.DomainServices.LoadingDataEventArgs e)
		{
			this.startTime = DateTime.Now;
			this.debugTextBox.Text = string.Format("--> Sending query to server: {0}\r\n"
				, e.Query.Query.ToString());
		}

		private void OnCustomersDataSourceLoadedData(object sender, Telerik.Windows.Controls.DomainServices.LoadedDataEventArgs e)
		{
			if (e.HasError)
			{
				// Have in mind that if you send a huge query to the server,
				// for example selecting hundreds of distinct values, you will
				// exceed the GET Request URI length limit and cause an exception.
				// When dealing with WCF RIA Service you might want to:
				// A. Disable the distinct values of RadGridView's FilteringControls by using 
				// GridViewBoundColumnBase.ShowDistinctFilter=false.
				// B. Mark the GetDistinctValues server method with [Query(hasSideEffects=true)] 
				// in order to cause a POST instead of GET.
				// C. Make sure the query is short enough in any other way you find reasonable.
				e.MarkErrorAsHandled();
				this.debugTextBox.Text = string.Format("<-- Server returned the following error: {0}\r\n"
					, e.Error);
			}
			else
			{
				TimeSpan elapsedTime = DateTime.Now - this.startTime;
				this.debugTextBox.Text += string.Format("<-- Server returned {0} entities {1} ms\r\n"
					, e.Entities.Count()
					, elapsedTime.Milliseconds);
			}
		}
	}
}
